import 'dart:async';
import "dart:io";
import "../../interactions/base.dart";
import './confirmation.dart';
import './selection.dart';
import './input.dart';
import 'package:console/console.dart';

///A Command Line Interface implementation
class CLInterface extends IOInterface {
  Stdin stdin;
  Stdout stdout;
  Stdout stderr;

  bool echoMode;
  bool lineMode;
  bool parent;

  Stream<List<int>> _stdinStream;

  CLInterface(this.stdin, this.stdout, this.stderr) {
    this._stdinStream = stdin.asBroadcastStream();
    this.parent = stdout.hasTerminal;
  }

  bool setUp([bool echoDelegated = false, lineDelegated = false]) {
    if (!parent) return true;
    try {
      // echoMode = stdin.echoMode;
      // lineMode = stdin.lineMode;
      // print("$echoMode - $lineMode");
      // print(StackTrace.current.toString());
      if (!echoDelegated) stdin.echoMode = false;
      if (!lineDelegated) stdin.lineMode = false;
    } on StdinException catch (e) {
      stderr.writeCharCode(5);
      if (!echoDelegated) {
        _commandParentShellEchoMode(false);
        return setUp(true);
      } else if (!lineDelegated) {
        _commandParentShellLineMode(false);
        return setUp(true, true);
      } else
        rethrow;
    } catch (e) {
      return false;
    }
    return true;
  }

  _commandParentShellEchoMode(bool mode) {
    if (mode == true)
      stderr.write("\x05\x01");
    else
      stderr.write("\x05\x03");
  }

  _commandParentShellLineMode(bool mode) {
    if (mode == true)
      stderr.write("\x05\x02");
    else
      stderr.write("\x05\x04");
  }

  bool tearDown([bool echoDelegated = false, lineDelegated = false]) {
    // print(StackTrace.current.toString());
    if (!parent) return true;
    try {
      // stdin.echoMode = echoMode;
      // stdin.lineMode = lineMode;
      if (!echoDelegated) stdin.echoMode = true;
      if (!lineDelegated) stdin.lineMode = true;
    } on StdinException catch (e) {
      stderr.writeCharCode(5);
      if (!echoDelegated) {
        _commandParentShellEchoMode(true);
        return tearDown(true);
      } else if (!lineDelegated) {
        _commandParentShellLineMode(true);
        return tearDown(true, true);
      } else
        rethrow;
    } catch (e) {
      return false;
    }
    return true;
  }

  String println([String contents]) {
    return print("$contents\n");
  }

  String print(String contents) {
    stdout.write(contents);
    return contents;
  }

  String printError(String message) {
    TextPen pen = new TextPen();
    pen.red();
    pen(message);
    pen.normal();
    stderr.writeln(pen.toString());
    return pen.toString();
  }

  Stream<String> get readChar =>
      _stdinStream.map((List<int> i) => new String.fromCharCodes(i));

  Stream<List<int>> get charCode => _stdinStream;

  Stream<String> get readInput async* {
    while (true) yield stdin.readLineSync();
  }

  int readByteSync() {
    // if (!parent) stderr.write("\x05\x05");
    return stdin.readByteSync();
  }
}

///An implementation for CLI interaction
class CLIInteractionsHandler extends InteractionsHandler {
  CLIInteractionsHandler(CLInterface clInterface) : super(clInterface);

  CLInterface get clInterface => this.ioInterface;

  @override
  Future<bool> askForConfirmation(String message, {bool defaultValue: true}) {
    CLIConfirmationInteraction interaction =
        new CLIConfirmationInteraction(clInterface, message, defaultValue);
    return interaction.execution();
  }

  @override
  Future<String> askForInput(String message, String checkRegExp) {
    CLIInputInteraction interaction =
        new CLIInputInteraction(clInterface, message, checkRegExp);
    return interaction.execution();
  }

  @override
  Future<String> askForSelection(String message, List<String> options) {
    CLISelectionInteraction interaction =
        new CLISelectionInteraction(clInterface, message, options);
    return interaction.execution();
  }
}

///Una clase al pedo que por ahi vaya a ser útil en algún momento, pero no se...
class CLIUtilsMixin {
  IOInterface ioInterface;
  CLInterface get clInterface => this.ioInterface;

  // Stream<bool> onEnterPress() {
  //   clInterface.charCode
  //       .where((List<int> input) => input == [10])
  //       .map((_) => true);
  // }
}
