import "../../interactions/base.dart";
import "./base.dart";
import 'dart:async';
import 'package:console/console.dart';

class CLIInputInteraction extends InputInteraction with CLIUtilsMixin {
  CLIInputInteraction(
      CLInterface clInterface, String message, String checkRegExp)
      : super(clInterface, message, checkRegExp) {
    if (this.ioInterface == null) {
      this.ioInterface = clInterface;
    }
  }

  @override
  Future<String> execution() async {
    TextPen pen = new TextPen();
    this.ioInterface.print("$message" + (message.endsWith(":") ? '' : ':'));
    String inputStr = "";
    String input;
    bool end = false;
    bool shouldCheckRE = checkRegExp != null;
    bool emphazise = false;
    if (!shouldCheckRE) checkRegExp = new RegExp(".*");
    while (!end) {
      int inputCharCode = clInterface.readByteSync();
      if (inputCharCode == 127) {
        //backspace
        if (inputStr.isNotEmpty)
          inputStr = inputStr.substring(0, inputStr.length - 1);
        else
          continue;
      } else if (inputCharCode == 10) {
        //Enter
        if (_matchAll(inputStr)) {
          end = true;
          continue;
        } else
          emphazise = true;
      } else {
        inputStr += new String.fromCharCode(inputCharCode);
      }
      if (shouldCheckRE) {
        pen.reset();
        if (_matchAll(inputStr))
          pen.green();
        else
          pen.red();
        pen(inputStr);
        pen.normal();
        input = "$pen";
      } else
        input = inputStr;
      String overwriteMessage = "$message" + (message.endsWith(":") ? '' : ':');
      if (emphazise) {
        emphazise = false;
        Console.overwriteLine(overwriteMessage);
        Console.setInverted(true);
        clInterface.print("$input");
        Console.setInverted(false);
      } else
        Console.overwriteLine("$overwriteMessage $input");
    }
    clInterface.stdout.writeln();
    return inputStr;
  }

  bool _matchAll(String input) {
    return input.contains(checkRegExp);
  }
}
