import 'dart:async';
import "../../interactions/base.dart";
import "./base.dart";
import 'package:console/console.dart';

class CLISelectionInteraction extends SelectionInteraction with CLIUtilsMixin {
  CLISelectionInteraction(
      CLInterface clInterface, String message, List<String> options)
      : super(clInterface, message, options) {
    if (this.ioInterface == null) {
      this.ioInterface = clInterface;
    }
  }

  Completer<String> _completer;
  int _election = 1;
  bool _started = false;

  _printOptions() {
    TextPen pen = new TextPen();
    int i = 1;
    for (String opt in options) {
      String line = "[$i] $opt";
      if (i == _election) {
        pen.reset();
        pen.blue();
        pen(line);
        pen.normal();
        clInterface.println(pen.toString());
      } else
        clInterface.println(line);
      i++;
    }
  }

  _eraseOptions() {
    for (var i in options) {
      Console.moveCursorUp();
      Console.eraseLine();
    }
  }

  Future<String> execution() async {
    clInterface.println("$message:");
    _printOptions();
    await for (List<int> inputCharCodes in clInterface.charCode) {
      if (inputCharCodes.first == 10) {
        //Enter
        clInterface.println();
        return options[_election - 1];
      } else if (inputCharCodes.length == 3) {
        if (inputCharCodes.sublist(0, 2).join() == "2791") {
          //Dart compares lists by identity :-/
          switch (inputCharCodes[2]) {
            case 65: //up
            case 68: //left
              if (_election > 1) _election--;
              break;
            case 66: //down
            case 67: //right
              if (_election < options.length) _election++;
              break;
            default:
              continue;
          }
          _eraseOptions();
          _printOptions();
        }
      } else if (inputCharCodes.first <= 57 && inputCharCodes.first > 48) {
        //Codes: 0 -> 48 ... 9 -> 57
        int number = inputCharCodes.first - 48;
        if (number <= options.length) {
          _election = number;
          _eraseOptions();
          _printOptions();
        }
      }
    }
  }

  // // Doesn't work unless the Keyboard calls are modified to be called just once
  // // Probably with an static setUp() method
  // Future<String> execution() {
  //   _completer = new Completer<String>();
  //   clInterface.print("$message:\n");
  //   _printOptions();
  //   Keyboard.echoUnhandledKeys = false;
  //   Keyboard.bindKey("up").listen((_) {
  //     if (_election > 1) {
  //       _election--;
  //       _eraseOptions();
  //       _printOptions();
  //     }
  //   });
  //
  //   Keyboard.bindKey("down").listen((_) {
  //     if (_election < options.length) {
  //       _election++;
  //       _eraseOptions();
  //       _printOptions();
  //     }
  //   });
  //
  //   List<String> codes = [];
  //   for (int i = 1; i < 9; i++) {
  //     // codes.add("${i + 48}"); the ASCII code
  //     codes.add("$i");
  //   }
  //
  //   Keyboard.bindKeys(codes).listen((String code) {
  //     int number = int.parse(code);
  //     if (number <= options.length) {
  //       _election = number;
  //       _eraseOptions();
  //       _printOptions();
  //     }
  //   });
  //
  //   Keyboard.bindKey("\n").listen((_) {
  //     clInterface.println();
  //     _completer.complete(options[_election - 1]);
  //   });
  //
  //   return _completer.future;
  // }
}
