import 'dart:async';

import './base.dart';
import '../generation/dependency.dart';

import 'package:recase/recase.dart';

class GeneratorsGenerator extends Generator {
  String get name => "generator";
  String get description => "scaffolds the code for a new generator";

  OverridingPolicy get overridePolicy => OverridingPolicy.NEVER;
  List<Dependency> get alwaysNeededDependencies => null;
  Map get startingVariables => null;

  Future execution() {
    askForInput("Generator name:", "[A-Z][a-z]*").then((String nombre) {
      ReCase reCase = new ReCase(nombre);
      variablesResolver["name"] = reCase.camelCase;
      variablesResolver["file_name"] = reCase.snakeCase;
    });
    askForInput("Generator description:", "[a-z ]*").then((String descripcion) {
      variablesResolver["description"] = descripcion;
    });
    askForSelection("Generator overriding policy:", ["ALWAYS", "ASK", "NEVER"])
        .then((String opt) {
      variablesResolver["override_policy"] = opt;
    });
    // variablesResolver["description"] = "descripcion";
    // variablesResolver["name"] = "nombre";
    // variablesResolver["generatorName"] = "Nombre";
    // variablesResolver["override_policy"] = "opt";
    askForSelection("Would this generator be...", [
      "PRIVATE: internal for this package only",
      "PUBLIC: usable anywhere from this package when imported"
    ]).then((String election) async {
      if (election.startsWith("PUBLIC")) {
        processFile("tool/generator+.dart", templates: {
          "new-generator":
              "new {{#pascal_case}}{{name}}Generator{{/pascal_case}}()"
        });
        processFile("lib/generator+/generators.dart",
            templates: {"new-generator": "{{ > generator }}\n"});
      } else {
        await generateTemplateDir(
            "package:generator_plus/src/templates/local_generator");
        processFile("tool/generator+.dart", templates: {
          "new-generator":
              "new {{#pascal_case}}{{name}}{{/pascal_case}}Generator()",
          "new-generator-import":
              'import "./generator+/generators/{{file_name}}.dart";'
        });
      }
    });
  }
}
