import './sequencer.dart';
import 'dart:async';
import 'package:generator_plus/src/interactions/base.dart';
import 'package:logging/logging.dart';

abstract class InteractionModule<T> extends GenerationStep<T> {
  Logger _logger;
  Logger get logger => _logger;
  InteractionsHandler interactionsHandler;
  String message;
  Completer _completer = new Completer();
  Future<T> get returnFuture;
}

class InteractionModuleInitializer {
  Logger logger;
  InteractionModuleInitializer(this.logger);

  void initialize(InteractionModule module) {
    module._logger = this.logger;
  }
}

class ConfirmationInteractionModule extends InteractionModule<bool> {
  bool defaultValue;
  ConfirmationInteractionModule(
      InteractionsHandler interactionsHandler, String message,
      {this.defaultValue: true}) {
    this.interactionsHandler = interactionsHandler;
    this.message = message;
  }
  @override
  bool execution() {
    interactionsHandler
        .askForConfirmation(message, defaultValue: this.defaultValue)
        .then((bool confirm) {
      _completer.complete(confirm);
    });
  }

  @override
  Future<bool> get returnFuture => _completer.future;
}

class SelectionInteractionModule extends InteractionModule<String> {
  List<String> options;
  SelectionInteractionModule(
      InteractionsHandler interactionsHandler, String message, this.options) {
    this.interactionsHandler = interactionsHandler;
    this.message = message;
  }
  @override
  String execution() {
    interactionsHandler
        .askForSelection(message, options)
        .then((String selection) {
      _completer.complete(selection);
    });
  }

  @override
  Future<String> get returnFuture => _completer.future;
}

class InputInteractionModule extends InteractionModule<String> {
  String checkRegExp;
  InputInteractionModule(InteractionsHandler interactionsHandler,
      String message, this.checkRegExp) {
    this.interactionsHandler = interactionsHandler;
    this.message = message;
  }
  @override
  String execution() {
    interactionsHandler.askForInput(message, checkRegExp).then((String input) {
      _completer.complete(input);
    });
  }

  @override
  Future<String> get returnFuture => _completer.future;
}
