import 'dart:async';
import 'dart:developer';
import 'dart:io';
import 'dart:isolate';

import 'package:path/path.dart' as path;
import 'package:generator_plus/src/common.dart';
import 'package:generator_plus/src/generation/base.dart';
import 'package:generator_plus/src/generation/new_dir.dart';
import 'package:generator_plus/src/generation/new_file.dart';

Future<String> resolvePath(String relativePath) async {
  if (relativePath.startsWith("package:")) {
    Uri uri = await Isolate.resolvePackageUri(Uri.parse(relativePath));
    if (uri.isScheme("file"))
      return uri.path;
    else if (uri.isScheme("http")) {
      //Hack to bypass the fucking 'pub run' replacings of file:/ for localhost:xxxx
      int index = relativePath.indexOf("/");
      String packageName = relativePath.substring(8, index);
      String path_rest = relativePath.substring(index);
      Map<String, String> envVars = Platform.environment;
      FileSystemEntity package = new File(
          "${envVars["HOME"]}/.pub-cache/hosted/pub.dartlang.org/$packageName/lib$path_rest");
      if (!package.existsSync())
        package = new Directory(
            "${envVars["HOME"]}/.pub-cache/hosted/pub.dartlang.org/$packageName/lib$path_rest");
      if (!package.existsSync()) {
        File pubPackages = new File("${getPackageRootPath()}.packages");
        String packages = pubPackages.readAsStringSync();
        RegExp match = new RegExp("$packageName:file:\/\/(.*)");
        String packagePath = match.firstMatch(packages)?.group(1);
        package = new Directory(packagePath ?? "");
      }
      if (!package.existsSync()) {
        // Hay q llamar a un Process con dart q ejecute esta misma funcion per
        // en vez de desde 'dart global run', desde dart directamente
        throw new Exception(
            "No se cómo resolver '$relativePath', vas a tener q laburarte un Process.run");
      }
      return package.path;
    }
  } else {
    return path.absolute(path.normalize(relativePath));
  }
}

Future<Directory> sourceTemplateDir(String templateRelativePath) async {
  if (templateRelativePath.startsWith("package:")) {
    String resolvedPath = await resolvePath(templateRelativePath);
    return new Directory(resolvedPath);
  } else if (path.isRelative(templateRelativePath))
    return new Directory(getPackageRootPath() + templateRelativePath);
  else
    throw new Exception("Don't know what to do with $templateRelativePath");
}

List<GenerationModule> templateGenerationModules(Directory sourceDir) {
  List<GenerationModule> ret = [];
  for (FileSystemEntity entity in sourceDir.listSync(recursive: true)) {
    String name = path.relative(entity.path, from: sourceDir.path);
    if (entity is File) {
      bool render = false;
      if (name.endsWith(".mustache")) {
        if (name.split('.').length > 2) {
          render = true;
          name = name.substring(0, name.length - 9);
        }
      }
      ret.add(new FileGenerationModule.fromExistingFile(entity, name,
          processInputWithMustache: render));
    } else if (entity is Directory) {
      // if (!entity.existsSync())
      ret.add(new DirGenerationModule(name, generateRecursively: false));
    }
  }
  return ret;
}
